<?php
/* Copyright (C) 2024	Easya Solutions	        <support@easya.solutions>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

use Luracast\Restler\RestException;


dol_include_once('/relationstierscontacts/class/relationtiers.class.php');

/**
 * API class for Advanced Dictionaries
 *
 * @access protected
 * @class  DolibarrApiAccess {@requires user,external}
 */
class RelationsTiersContactsApi extends DolibarrApi
{
	/**
	 *
	 * @var array   $FIELDS     Mandatory fields, checked when create and update object
	 */
	public static $FIELDS = array();


	/**
	 *  Constructor
	 */
	public function __construct()
	{
		global $db;

		$this->db = $db;

		//require_once DOL_DOCUMENT_ROOT.'/categories/class/categorie.class.php';
	}

	/**
	 *  List of "Relation Tiers"
	 *
	 * Get a list of "Relation Tiers"
	 *
	 * @url	GET /
	 *
	 * @param	string	$sortfield			Sort field
	 * @param	string	$sortorder			Sort order
	 * @param	int		$limit				Limit for list
	 * @param	int		$page				Page number
	 * @param	string	$thirdparty_ids		Third-party ids to filter contacts of (example '1' or '1,2,3') {@pattern /^[0-9,]*$/i}
	 * @param	int		$category			Use this param to filter list by category
	 * @param	string	$sqlfilters			Other criteria to filter answers separated by a comma. Syntax example "(t.ref:like:'SO-%') and (t.date_creation:<:'20160101')"
	 * @return	RelationTiers[]				Array of contact objects
	 *
	 * @throws  400		RestException       Error when validating parameter 'sqlfilters'
	 * @throws  403		RestException       Insufficient rights
	 * @throws  404		RestException       Contact not found
	 * @throws  503		RestException       Error when retrieve list
	 */
	public function index($sortfield = 't.rowid', $sortorder = 'ASC', $limit = 100, $page = 0, $thirdparty_ids = '', $category = 0, $sqlfilters = '')
	{
		global $db, $conf;

		$obj_ret = array();

		if (!DolibarrApiAccess::$user->hasRight('societe', 'contact', 'lire')) {
			throw new RestException(403, 'No permission to read contacts');
		}

		// case of external user, $thirdparty_ids param is ignored and replaced by user's socid
		$socids = DolibarrApiAccess::$user->socid ?: $thirdparty_ids;

		// If the internal user must only see his customers, force searching by him
		$search_sale = 0;
		if (!DolibarrApiAccess::$user->hasRight('societe', 'client', 'voir') && !$socids) {
			$search_sale = DolibarrApiAccess::$user->id;
		}

		$sql = "SELECT t.rowid";
		$sql .= " FROM " . $this->db->prefix() . "relationtiers as t";
		$sql .= " LEFT JOIN " . $this->db->prefix() . "socpeople as sp ON sp.rowid = t.fk_socpeople";
		//$sql .= " LEFT JOIN " . $this->db->prefix() . "socpeople_extrafields as spef on (sp.rowid = spef.fk_object)";
		//$sql .= " LEFT JOIN " . $this->db->prefix() . "c_relationtiers as crt ON crt.rowid = t.fk_c_relationtiers";
		// select contacts of given category
		if ($category > 0) {
			$sql .= " INNER JOIN ".$this->db->prefix()."categorie_contact as cc ON cc.fk_socpeople = t.fk_socpeople AND cc.fk_categorie = ".((int) $category);
		}
		$sql .= " WHERE sp.entity IN (".getEntity('contact').")";
		if ($socids) {
			$sql .= " AND t.fk_soc IN (".$this->db->sanitize($socids).")";
		}
		// Search on sale representative
		if ($search_sale && $search_sale != '-1') {
			if ($search_sale == -2) {
				$sql .= " AND NOT EXISTS (SELECT sc.fk_soc FROM ".$this->db->prefix()."societe_commerciaux as sc WHERE sc.fk_soc = t.fk_soc)";
			} elseif ($search_sale > 0) {
				$sql .= " AND EXISTS (SELECT sc.fk_soc FROM ".$this->db->prefix()."societe_commerciaux as sc WHERE sc.fk_soc = t.fk_soc AND sc.fk_user = ".((int) $search_sale).")";
			}
		}
		// Add sql filters
		if ($sqlfilters) {
			$errormessage = '';
			$sql .= forgeSQLFromUniversalSearchCriteria($sqlfilters, $errormessage);
			if ($errormessage) {
				throw new RestException(400, 'Error when validating parameter sqlfilters -> '.$errormessage);
			}
		}

		$sql .= $this->db->order($sortfield, $sortorder);

		if ($limit) {
			if ($page < 0) {
				$page = 0;
			}
			$offset = $limit * $page;

			$sql .= $this->db->plimit($limit + 1, $offset);
		}
		$result = $this->db->query($sql);
		if ($result) {
			$num = $this->db->num_rows($result);
			$min = min($num, ($limit <= 0 ? $num : $limit));
			$i = 0;
			while ($i < $min) {
				$obj = $this->db->fetch_object($result);
				$relationTiersStatic = new RelationTiers($this->db);
				if ($relationTiersStatic->fetch($obj->rowid, true)) {
					$obj_ret[] = $this->_cleanObjectDatas($relationTiersStatic);
				}

				$i++;
			}
		} else {
			throw new RestException(503, 'Error when retrieve relation tiers : '.$sql);
		}
		if (!count($obj_ret)) {
			throw new RestException(404, 'Relation tiers not found');
		}
		return $obj_ret;
	}

	/**
	 * Clean sensible object datas
	 *
	 * @param   Object  $object     Object to clean
	 * @return  Object              Object with cleaned properties
	 */
	protected function _cleanObjectDatas($object)
	{
		// clean linked contact object
		$contact = null;
		if (isset($object->contact)) {
			$contact = $object->contact;
			$contact = parent::_cleanObjectDatas($contact);

			unset($contact->total_ht);
			unset($contact->total_tva);
			unset($contact->total_localtax1);
			unset($contact->total_localtax2);
			unset($contact->total_ttc);

			unset($object->note);
			unset($object->lines);
			unset($object->thirdparty);
		}

		// clean relation tiers objet
		$object = parent::_cleanObjectDatas($object);
		$object->contact = $contact;

		unset($object->total_ht);
		unset($object->total_tva);
		unset($object->total_localtax1);
		unset($object->total_localtax2);
		unset($object->total_ttc);

		unset($object->note);
		unset($object->lines);
		unset($object->thirdparty);

		return $object;
	}
}
